@@ -368,6 +368,36 @@ static DBusMessage *agent_report_error(DBusConnection *connection,
 	return NULL;
 }
 
+static DBusMessage *agent_report_peer_error(DBusConnection *connection,
+					DBusMessage *message, void *user_data)
+{
+	struct agent_data *request = user_data;
+	char *path, *peer, *error;
+	DBusMessageIter iter;
+
+	if (handle_message(message, request,
+				agent_report_peer_error) == false)
+		return NULL;
+
+	dbus_message_iter_init(message, &iter);
+
+	dbus_message_iter_get_basic(&iter, &path);
+	peer = strip_path(path);
+
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_get_basic(&iter, &error);
+
+	__connmanctl_save_rl();
+	fprintf(stdout, "Agent ReportPeerError %s\n", peer);
+	fprintf(stdout, "  %s\n", error);
+	__connmanctl_redraw_rl();
+
+	request->message = dbus_message_ref(message);
+	__connmanctl_agent_mode("Retry (yes/no)? ",
+				report_error_return, request);
+	return NULL;
+}
+
 static void request_input_next(struct agent_data *request)
 {
 	int i;
@@ -462,37 +492,16 @@ static void request_input_string_return(char *input, void *user_data)
 	request_input_next(request);
 }
 
-static DBusMessage *agent_request_input(DBusConnection *connection,
-		DBusMessage *message, void *user_data)
+static void parse_agent_request(struct agent_data *request,
+						DBusMessageIter *iter)
 {
-	struct agent_data *request = user_data;
-	DBusMessageIter iter, dict, entry, variant;
-	char *service, *str, *field;
-	DBusMessageIter dict_entry, field_entry, field_value;
-	char *argument, *value, *attr_type = NULL;
-
+	DBusMessageIter dict, entry, variant, dict_entry;
+	DBusMessageIter field_entry, field_value;
+	char *field, *argument, *value;
+	char *attr_type = NULL;
 	int i;
 
-	if (handle_message(message, request, agent_request_input) == false)
-		return NULL;
-
-	dbus_message_iter_init(message, &iter);
-
-	dbus_message_iter_get_basic(&iter, &str);
-	service = strip_path(str);
-
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &dict);
-
-	__connmanctl_save_rl();
-	if (strcmp(request->interface, AGENT_INTERFACE) == 0)
-		fprintf(stdout, "Agent RequestInput %s\n", service);
-	else
-		fprintf(stdout, "VPN Agent RequestInput %s\n", service);
-	__connmanctl_dbus_print(&dict, "  ", " = ", "\n");
-	fprintf(stdout, "\n");
-
-	dbus_message_iter_recurse(&iter, &dict);
+	dbus_message_iter_recurse(iter, &dict);
 
 	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
 
@@ -536,6 +545,76 @@ static DBusMessage *agent_request_input(DBusConnection *connection,
 
 		dbus_message_iter_next(&dict);
 	}
+}
+
+static DBusMessage *agent_request_input(DBusConnection *connection,
+		DBusMessage *message, void *user_data)
+{
+	struct agent_data *request = user_data;
+	DBusMessageIter iter, dict;
+	char *service, *str;
+
+	if (handle_message(message, request, agent_request_input) == false)
+		return NULL;
+
+	dbus_message_iter_init(message, &iter);
+
+	dbus_message_iter_get_basic(&iter, &str);
+	service = strip_path(str);
+
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &dict);
+
+	__connmanctl_save_rl();
+	if (strcmp(request->interface, AGENT_INTERFACE) == 0)
+		fprintf(stdout, "Agent RequestInput %s\n", service);
+	else
+		fprintf(stdout, "VPN Agent RequestInput %s\n", service);
+	__connmanctl_dbus_print(&dict, "  ", " = ", "\n");
+	fprintf(stdout, "\n");
+
+	parse_agent_request(request, &iter);
+
+	request->reply = dbus_message_new_method_return(message);
+	dbus_message_iter_init_append(request->reply, &request->iter);
+
+	dbus_message_iter_open_container(&request->iter, DBUS_TYPE_ARRAY,
+                        DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+                        DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+                        DBUS_DICT_ENTRY_END_CHAR_AS_STRING,
+			&request->dict);
+
+	request_input_next(request);
+
+	return NULL;
+}
+
+static DBusMessage *
+agent_request_peer_authorization(DBusConnection *connection,
+					DBusMessage *message, void *user_data)
+{
+	struct agent_data *request = user_data;
+	DBusMessageIter iter, dict;
+	char *peer, *str;
+
+	if (handle_message(message, request, agent_request_peer_authorization)
+								== false)
+		return NULL;
+
+	dbus_message_iter_init(message, &iter);
+
+	dbus_message_iter_get_basic(&iter, &str);
+	peer = strip_path(str);
+
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &dict);
+
+	__connmanctl_save_rl();
+	fprintf(stdout, "Agent RequestPeerAuthorization %s\n", peer);
+	__connmanctl_dbus_print(&dict, "  ", " = ", "\n");
+	fprintf(stdout, "\n");
+
+	parse_agent_request(request, &iter);
 
 	request->reply = dbus_message_new_method_return(message);
 	dbus_message_iter_init_append(request->reply, &request->iter);
@@ -562,11 +641,20 @@ static const GDBusMethodTable agent_methods[] = {
 				GDBUS_ARGS({ "service", "o" },
 					{ "error", "s" }),
 				NULL, agent_report_error) },
+	{ GDBUS_ASYNC_METHOD("ReportPeerError",
+				GDBUS_ARGS({ "peer", "o" },
+					{ "error", "s" }),
+				NULL, agent_report_peer_error) },
 	{ GDBUS_ASYNC_METHOD("RequestInput",
 				GDBUS_ARGS({ "service", "o" },
 					{ "fields", "a{sv}" }),
 				GDBUS_ARGS({ "fields", "a{sv}" }),
 				agent_request_input) },
+	{ GDBUS_ASYNC_METHOD("RequestPeerAuthorization",
+				GDBUS_ARGS({ "peer", "o" },
+					{ "fields", "a{sv}" }),
+				GDBUS_ARGS({ "fields", "a{sv}" }),
+				agent_request_peer_authorization) },
 	{ },
 };
 
@@ -588,6 +676,13 @@ static int agent_register_return(DBusMessageIter *iter, const char *error,
 	return -EINPROGRESS;
 }
 
+static void append_path(DBusMessageIter *iter, void *user_data)
+{
+	const char *path = user_data;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH, &path);
+}
+
 int __connmanctl_agent_register(DBusConnection *connection)
 {
 	char *path = agent_path();
@@ -609,8 +704,7 @@ int __connmanctl_agent_register(DBusConnection *connection)
 
 	result = __connmanctl_dbus_method_call(connection, CONNMAN_SERVICE,
 			CONNMAN_PATH, "net.connman.Manager", "RegisterAgent",
-			agent_register_return, connection,
-			DBUS_TYPE_OBJECT_PATH, &path, DBUS_TYPE_INVALID);
+			agent_register_return, connection, append_path, path);
 
 	if (result != -EINPROGRESS) {
 		g_dbus_unregister_interface(connection, agent_path(),
@@ -650,8 +744,7 @@ int __connmanctl_agent_unregister(DBusConnection *connection)
 
 	result = __connmanctl_dbus_method_call(connection, CONNMAN_SERVICE,
 			CONNMAN_PATH, "net.connman.Manager", "UnregisterAgent",
-			agent_unregister_return, NULL,
-			DBUS_TYPE_OBJECT_PATH, &path, DBUS_TYPE_INVALID);
+			agent_unregister_return, NULL, append_path, path);
 
 	if (result != -EINPROGRESS)
 		fprintf(stderr, "Error: Failed to unregister Agent\n");
@@ -714,8 +807,8 @@ int __connmanctl_vpn_agent_register(DBusConnection *connection)
 
 	result = __connmanctl_dbus_method_call(connection, VPN_SERVICE,
 			VPN_PATH, "net.connman.vpn.Manager", "RegisterAgent",
-			vpn_agent_register_return, connection,
-			DBUS_TYPE_OBJECT_PATH, &path, DBUS_TYPE_INVALID);
+			vpn_agent_register_return, connection, append_path,
+			path);
 
 	if (result != -EINPROGRESS) {
 		g_dbus_unregister_interface(connection, agent_path(),
@@ -756,8 +849,7 @@ int __connmanctl_vpn_agent_unregister(DBusConnection *connection)
 
 	result = __connmanctl_dbus_method_call(connection, VPN_SERVICE,
 			VPN_PATH, "net.connman.vpn.Manager", "UnregisterAgent",
-			vpn_agent_unregister_return, NULL,
-			DBUS_TYPE_OBJECT_PATH, &path, DBUS_TYPE_INVALID);
+			vpn_agent_unregister_return, NULL, append_path, path);
 
 	if (result != -EINPROGRESS)
 		fprintf(stderr, "Error: Failed to unregister VPN Agent\n");
