@@ -2,7 +2,7 @@
  *
  *  DHCP client library with GLib integration
  *
- *  Copyright (C) 2009-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2009-2014  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -47,11 +47,11 @@
 #include "common.h"
 #include "ipv4ll.h"
 
-#define DISCOVER_TIMEOUT 3
-#define DISCOVER_RETRIES 10
+#define DISCOVER_TIMEOUT 5
+#define DISCOVER_RETRIES 6
 
-#define REQUEST_TIMEOUT 3
-#define REQUEST_RETRIES 5
+#define REQUEST_TIMEOUT 5
+#define REQUEST_RETRIES 3
 
 typedef enum _listen_mode {
 	L_NONE,
@@ -105,7 +105,6 @@ struct _GDHCPClient {
 	guint t2_timeout;
 	guint lease_timeout;
 	guint listener_watch;
-	GIOChannel *listener_channel;
 	GList *require_list;
 	GList *request_list;
 	GHashTable *code_value_hash;
@@ -485,13 +484,13 @@ static int send_request(GDHCPClient *dhcp_client)
 
 	add_send_options(dhcp_client, &packet);
 
-	if (dhcp_client->state == RENEWING) {
+	if (dhcp_client->state == RENEWING || dhcp_client->state == REBINDING)
 		packet.ciaddr = htonl(dhcp_client->requested_ip);
 
+	if (dhcp_client->state == RENEWING)
 		return dhcp_send_kernel_packet(&packet,
 				dhcp_client->requested_ip, CLIENT_PORT,
 				dhcp_client->server_ip, SERVER_PORT);
-	}
 
 	return dhcp_send_raw_packet(&packet, INADDR_ANY, CLIENT_PORT,
 					INADDR_BROADCAST, SERVER_PORT,
@@ -1163,7 +1162,6 @@ GDHCPClient *g_dhcp_client_new(GDHCPType type,
 	get_interface_mac_address(ifindex, dhcp_client->mac_address);
 
 	dhcp_client->listener_sockfd = -1;
-	dhcp_client->listener_channel = NULL;
 	dhcp_client->listen_mode = L_NONE;
 	dhcp_client->ref_count = 1;
 	dhcp_client->type = type;
@@ -1526,7 +1524,6 @@ static int switch_listening_mode(GDHCPClient *dhcp_client,
 	if (dhcp_client->listen_mode != L_NONE) {
 		if (dhcp_client->listener_watch > 0)
 			g_source_remove(dhcp_client->listener_watch);
-		dhcp_client->listener_channel = NULL;
 		dhcp_client->listen_mode = L_NONE;
 		dhcp_client->listener_sockfd = -1;
 		dhcp_client->listener_watch = 0;
@@ -1563,7 +1560,6 @@ static int switch_listening_mode(GDHCPClient *dhcp_client,
 
 	dhcp_client->listen_mode = listen_mode;
 	dhcp_client->listener_sockfd = listener_sockfd;
-	dhcp_client->listener_channel = listener_channel;
 
 	g_io_channel_set_close_on_unref(listener_channel, TRUE);
 	dhcp_client->listener_watch =
@@ -1571,7 +1567,7 @@ static int switch_listening_mode(GDHCPClient *dhcp_client,
 				G_IO_IN | G_IO_NVAL | G_IO_ERR | G_IO_HUP,
 						listener_event, dhcp_client,
 								NULL);
-	g_io_channel_unref(dhcp_client->listener_channel);
+	g_io_channel_unref(listener_channel);
 
 	return 0;
 }
@@ -1643,13 +1639,13 @@ static gboolean start_expire(gpointer user_data)
 	/*remove all timeouts if they are set*/
 	remove_timeouts(dhcp_client);
 
+	restart_dhcp(dhcp_client, 0);
+
 	/* ip need to be cleared */
 	if (dhcp_client->lease_lost_cb)
 		dhcp_client->lease_lost_cb(dhcp_client,
 				dhcp_client->lease_lost_data);
 
-	restart_dhcp(dhcp_client, 0);
-
 	return false;
 }
 
@@ -2370,6 +2366,12 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 			g_free(dhcp_client->assigned_ip);
 			dhcp_client->assigned_ip = get_ip(packet.yiaddr);
 
+			if (dhcp_client->state == REBOOTING) {
+				option = dhcp_get_option(&packet,
+							DHCP_SERVER_ID);
+				dhcp_client->server_ip = get_be32(option);
+			}
+
 			/* Address should be set up here */
 			if (dhcp_client->lease_available_cb)
 				dhcp_client->lease_available_cb(dhcp_client,
@@ -2831,8 +2833,6 @@ void g_dhcp_client_stop(GDHCPClient *dhcp_client)
 		dhcp_client->listener_watch = 0;
 	}
 
-	dhcp_client->listener_channel = NULL;
-
 	dhcp_client->retry_times = 0;
 	dhcp_client->ack_retry_times = 0;
 
