@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2012-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -804,7 +804,8 @@ static void dad_reply(struct nd_neighbor_advert *reply,
 			service = __connman_service_lookup_from_index(
 								data->ifindex);
 			network = __connman_service_get_network(service);
-			data->callback(network, status, NULL);
+			if (network)
+				data->callback(network, status, NULL);
 		}
 	}
 
@@ -1398,7 +1399,7 @@ int __connman_dhcpv6_start_renew(struct connman_network *network,
 							dhcpv6_cb callback)
 {
 	struct connman_dhcpv6 *dhcp;
-	uint32_t T1, T2;
+	uint32_t T1, T2, delta;
 	time_t started, current, expired;
 
 	dhcp = g_hash_table_lookup(network_table, network);
@@ -1421,11 +1422,13 @@ int __connman_dhcpv6_start_renew(struct connman_network *network,
 		/* RFC 3315, 22.4 */
 		return 0;
 
-	if (T1 == 0)
+	if (T1 == 0) {
 		/* RFC 3315, 22.4
 		 * Client can choose the timeout.
 		 */
-		T1 = 1800;
+		T1 = (expired - started) / 2;
+		T2 = (expired - started) / 10 * 8;
+	}
 
 	dhcp->callback = callback;
 
@@ -1436,22 +1439,23 @@ int __connman_dhcpv6_start_renew(struct connman_network *network,
 	if (T2 != 0xffffffff && T2 > 0) {
 		if ((unsigned)current >= (unsigned)started + T2) {
 			/* RFC 3315, chapter 18.1.3, start rebind */
-			DBG("rebind after %d secs", T2);
+			DBG("start rebind immediately");
 
-			dhcp->timeout = g_timeout_add_seconds(T2, start_rebind,
+			dhcp->timeout = g_timeout_add_seconds(0, start_rebind,
 							dhcp);
 
 		} else if ((unsigned)current < (unsigned)started + T1) {
-			DBG("renew after %d secs", T1);
+			delta = started + T1 - current;
+			DBG("renew after %d secs", delta);
 
-			dhcp->timeout = g_timeout_add_seconds(T1, start_renew,
-							dhcp);
+			dhcp->timeout = g_timeout_add_seconds(delta,
+					start_renew, dhcp);
 		} else {
-			DBG("rebind after %d secs", T2 - T1);
+			delta = started + T2 - current;
+			DBG("rebind after %d secs", delta);
 
-			dhcp->timeout = g_timeout_add_seconds(T2 - T1,
-							start_rebind,
-							dhcp);
+			dhcp->timeout = g_timeout_add_seconds(delta,
+					start_rebind, dhcp);
 		}
 	}
 
