@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -1380,18 +1380,29 @@ static void dump_ipt_replace(struct ipt_replace *repl)
 static int iptables_get_entries(struct connman_iptables *table)
 {
 	socklen_t entry_size;
+	int err;
 
 	entry_size = sizeof(struct ipt_get_entries) + table->info->size;
 
-	return getsockopt(table->ipt_sock, IPPROTO_IP, IPT_SO_GET_ENTRIES,
+	err = getsockopt(table->ipt_sock, IPPROTO_IP, IPT_SO_GET_ENTRIES,
 				table->blob_entries, &entry_size);
+	if (err < 0)
+		return -errno;
+
+	return 0;
 }
 
 static int iptables_replace(struct connman_iptables *table,
 					struct ipt_replace *r)
 {
-	return setsockopt(table->ipt_sock, IPPROTO_IP, IPT_SO_SET_REPLACE, r,
-			 sizeof(*r) + r->size);
+	int err;
+
+	err = setsockopt(table->ipt_sock, IPPROTO_IP, IPT_SO_SET_REPLACE, r,
+			sizeof(*r) + r->size);
+	if (err < 0)
+		return -errno;
+
+	return 0;
 }
 
 static int add_entry(struct ipt_entry *entry, int builtin, unsigned int hook,
@@ -1584,11 +1595,6 @@ static struct xtables_target *prepare_target(struct connman_iptables *table,
 		else if (is_user_defined) {
 			struct connman_iptables_entry *target_rule;
 
-			if (!chain_head) {
-				g_free(xt_t->t);
-				return NULL;
-			}
-
 			target_rule = chain_head->next->data;
 			target->verdict = target_rule->offset;
 		}
